// 
//
//////////////////////////////////////////////////////////////////////
#ifndef _PROBABILITY_H
#define _PROBABILITY_H

#include <stdio.h>
#include <math.h>

//正态分布中的常数
#define _PROBABILITY_NORM_A1 0.0705230784
#define _PROBABILITY_NORM_A2 0.0422820123
#define _PROBABILITY_NORM_A3 0.0092705272
#define _PROBABILITY_NORM_A4 0.0001520143
#define _PROBABILITY_NORM_A5 0.0002765672
#define _PROBABILITY_NORM_A6 0.0000430638


//正态分布反函数中的常数
#define _PROBABILITY_RENORM_B0 1.570796288
#define _PROBABILITY_RENORM_B1 0.3706987906e-1
#define _PROBABILITY_RENORM_B2 -0.8364353589e-3
#define _PROBABILITY_RENORM_B3 -0.2250947176e-3
#define _PROBABILITY_RENORM_B4 0.6841218299e-5
#define _PROBABILITY_RENORM_B5 0.5824238515e-5
#define _PROBABILITY_RENORM_B6 -0.1045274970e-5
#define _PROBABILITY_RENORM_B7 0.8360937017e-7
#define _PROBABILITY_RENORM_B8 -0.3231081277e-8
#define _PROBABILITY_RENORM_B9 0.3657763036e-10
#define _PROBABILITY_RENORM_B10 0.6936233982e-12


//////////////////////////////////////////////////////////////////////////
//概率计算

	// 伽玛函数
	double probability_gam(int n);

	//  正态分布的分布函数值: p(-∞,u)
    	double probability_norm(double u);
	
    	//  正态分布的反函数, p(-∞,u) ; 已知p, 返回u
	double probability_re_norm(double q);
    
	//  chi2分布的分布函数：p(0,x)
    	double probability_chi2(int n,double x,double *f);
	
	//  chi2分布的反函数：p=chi2(0,x),已知x，反求p
	double probability_re_chi2(int n,double p);
    
	//B分布函数值，F分布和t分布函数值计算将调用本函数
	double probability_B(int n1,int n2, double x,double *Ux);

    	//  F分布函数值:区间(0,x)上的概率p, f-密度值
	double probability_F(int n1,int n2, double x,double *f);

    	//  F分布的反函数：p=F(0,x), 已知p,反求x
	double probability_re_F(int n1,int n2,double p);
	
	//  t分布的分布函数值
	double probability_t(int n, double x,double *f);

	//  t分布的反函数: p(-∞,u)=p; 已知p, 返回u
	double probability_re_t(int n,double p); 
			
///////////////////////////////////////////////////////


#endif // _PROBABILITY_H
